@@ -165,7 +165,12 @@ class ModflowNwt(Package):
     extension : list string
         Filename extension (default is 'nwt')
     unitnumber : int
-        File unit number (default is 32).
+        File unit number (default is None).
+    filenames : str or list of str
+        Filenames to use for the package. If filenames=None the package name
+        will be created using the model name and package extension. If a
+        single string is passed the package will be set to the string.
+        Default is None.
 
     Attributes
     ----------
@@ -197,15 +202,37 @@ class ModflowNwt(Package):
                  msdr=15, \
                  iacl=2, norder=1, level=5, north=7, iredsys=0, rrctols=0.0, \
                  idroptol=1, epsrn=1.e-4, hclosexmd=1e-4, mxiterxmd=50, \
-                 extension='nwt', unitnumber=32):
+                 extension='nwt', unitnumber=None, filenames=None):
 
         if model.version != 'mfnwt':
             err = 'Error: model version must be mfnwt to use NWT package'
             raise Exception(err)
 
-        Package.__init__(self, model, extension, 'NWT',
-                         unitnumber)  # Call ancestor's init to set self.parent, extension, name and unit number
-        self.heading = '# NWT for MODFLOW-NWT, generated by Flopy.'
+        # set default unit number of one is not specified
+        if unitnumber is None:
+            unitnumber = ModflowNwt.defaultunit()
+
+        # set filenames
+        if filenames is None:
+            filenames = [None]
+        elif isinstance(filenames, str):
+            filenames = [filenames]
+
+        # Fill namefile items
+        name = [ModflowNwt.ftype()]
+        units = [unitnumber]
+        extra = ['']
+
+        # set package name
+        fname = [filenames[0]]
+
+        # Call ancestor's init to set self.parent, extension, name and unit number
+        Package.__init__(self, model, extension=extension, name=name,
+                         unit_number=units, extra=extra, filenames=fname)
+
+        self.heading = '# {} package for '.format(self.name[0]) + \
+                       ' {}, '.format(model.version_types[model.version]) + \
+                       'generated by Flopy.'
         self.url = 'nwt_newton_solver.htm'
         self.headtol = headtol
         self.fluxtol = fluxtol
@@ -352,7 +379,7 @@ class ModflowNwt(Package):
             if line[0] != '#':
                 break
         # dataset 1
-        ifrfm = model.free_format_input
+        ifrfm = True #model.free_format_input
 
         vars = (("headtol", float), ("fluxtol", float), ("maxiterout", int),
                 ("thickfact", float), ("linmeth", int), ("iprnwt", int),
@@ -418,10 +445,34 @@ class ModflowNwt(Package):
                 for idx, (k, c) in enumerate(lindict.items()):
                     t.append(line[idx * 10:(idx + 1) * 10])
             for idx, (k, c) in enumerate(lindict.items()):
-                kwargs[k] = c(t[idx])
+                # forgive missing value for MXITERXMD (last value)
+                # (apparently NWT runs without it)
+                if len(t) > 0:
+                    kwargs[k] = c(t.pop(0))
 
         # close the open file
         f.close()
 
+        # determine specified unit number
+        # set package unit number
+        unitnumber = None
+        filenames = [None]
+        if ext_unit_dict is not None:
+            unitnumber, filenames[0] = \
+                model.get_ext_dict_attr(ext_unit_dict,
+                                        filetype=ModflowNwt.ftype())
+        kwargs['unitnumber'] = unitnumber
+        kwargs['filenames'] = filenames
+
         # create and return an instance of the nwt class
         return ModflowNwt(model, **kwargs)
+
+
+    @staticmethod
+    def ftype():
+        return 'NWT'
+
+
+    @staticmethod
+    def defaultunit():
+        return 32
